@@ -45,13 +45,16 @@ COV_KEY="${MSYSTEM}.${PYVER}"
 mkdir -p "${COV_DIR}"
 export COVERAGE_FILE="${COV_DIR}/.coverage.${COV_KEY}"
 
+# https://docs.python.org/3/using/cmdline.html#envvar-PYTHONDEVMODE
+export PYTHONDEVMODE=1
+
 $PYTHON setup.py build_tests
 MSYSTEM= $PYTHON -m coverage run tests/runtests.py
 
-curl -O -J -L "https://github.com/linux-test-project/lcov/releases/download/v1.13/lcov-1.13.tar.gz"
-tar -xvzf lcov-1.13.tar.gz
+curl -O -J -L "https://github.com/linux-test-project/lcov/archive/master.tar.gz"
+tar -xvzf lcov-master.tar.gz
 
-./lcov-1.13/bin/lcov \
+./lcov-master/bin/lcov \
     --rc lcov_branch_coverage=1 --no-external \
     --directory . --capture --output-file \
     "${COV_DIR}/${COV_KEY}.lcov"
