@@ -196,8 +196,6 @@ pyg_value_array_from_pyobject(GValue *value,
     for (i = 0; i < len; ++i) {
         PyObject *item = PySequence_GetItem(obj, i);
         GType type;
-        GValue item_value = { 0, };
-        int status;
 
         if (! item) {
             PyErr_Clear();
@@ -219,20 +217,27 @@ pyg_value_array_from_pyobject(GValue *value,
             }
         }
 
-        g_value_init(&item_value, type);
-        status = (pspec && pspec->element_spec)
-                 ? pyg_param_gvalue_from_pyobject(&item_value, item, pspec->element_spec)
-                 : pyg_value_from_pyobject(&item_value, item);
-        Py_DECREF(item);
+        if (type == G_TYPE_VALUE) {
+            const GValue * item_value = pyg_boxed_get(item, GValue);
+            g_value_array_append(value_array, item_value);
+        } else {
+            GValue item_value = { 0, };
+            int status;
 
-        if (status == -1) {
-            g_value_array_free(value_array);
+            g_value_init(&item_value, type);
+            status = (pspec && pspec->element_spec)
+                ? pyg_param_gvalue_from_pyobject(&item_value, item, pspec->element_spec)
+                : pyg_value_from_pyobject(&item_value, item);
+            Py_DECREF(item);
+
+            if (status == -1) {
+                g_value_array_free(value_array);
+                g_value_unset(&item_value);
+                return -1;
+            }
+            g_value_array_append(value_array, &item_value);
             g_value_unset(&item_value);
-            return -1;
         }
-
-        g_value_array_append(value_array, &item_value);
-        g_value_unset(&item_value);
     }
 
     g_value_take_boxed(value, value_array);
@@ -312,7 +317,6 @@ pyg_array_from_pyobject(GValue *value,
 int
 pyg_value_from_pyobject_with_error(GValue *value, PyObject *obj)
 {
-    PyObject *tmp;
     GType value_type = G_VALUE_TYPE(value);
 
     switch (G_TYPE_FUNDAMENTAL(value_type)) {
@@ -339,50 +343,23 @@ pyg_value_from_pyobject_with_error(GValue *value, PyObject *obj)
         }
         break;
     case G_TYPE_CHAR:
-        if (PYGLIB_PyLong_Check(obj)) {
-            glong val;
-            val = PYGLIB_PyLong_AsLong(obj);
-            if (val >= -128 && val <= 127)
-                g_value_set_schar(value, (gchar) val);
-            else
-                return -1;
-        }
-#if PY_VERSION_HEX < 0x03000000
-        else if (PyString_Check(obj)) {
-            g_value_set_schar(value, PyString_AsString(obj)[0]);
-        }
-#endif
-        else if (PyUnicode_Check(obj)) {
-            tmp = PyUnicode_AsUTF8String(obj);
-            g_value_set_schar(value, PYGLIB_PyBytes_AsString(tmp)[0]);
-            Py_DECREF(tmp);
-        } else {
-            PyErr_SetString(PyExc_TypeError, "Cannot convert to TYPE_CHAR");
+    {
+        gint8 temp;
+        if (pygi_gschar_from_py (obj, &temp)) {
+            g_value_set_schar (value, temp);
+            return 0;
+        } else
             return -1;
-        }
-
-        break;
+    }
     case G_TYPE_UCHAR:
-        if (PYGLIB_PyLong_Check(obj)) {
-            glong val;
-            val = PYGLIB_PyLong_AsLong(obj);
-            if (val >= 0 && val <= 255)
-                g_value_set_uchar(value, (guchar) val);
-            else
-                return -1;
-#if PY_VERSION_HEX < 0x03000000
-        } else if (PyString_Check(obj)) {
-            g_value_set_uchar(value, PyString_AsString(obj)[0]);
-#endif
-        } else if (PyUnicode_Check(obj)) {
-            tmp = PyUnicode_AsUTF8String(obj);
-            g_value_set_uchar(value, PYGLIB_PyBytes_AsString(tmp)[0]);
-            Py_DECREF(tmp);
-        } else {
-            PyErr_Clear();
+    {
+        guchar temp;
+        if (pygi_guchar_from_py (obj, &temp)) {
+            g_value_set_uchar (value, temp);
+            return 0;
+        } else
             return -1;
-        }
-        break;
+    }
     case G_TYPE_BOOLEAN:
     {
         gboolean temp;
@@ -487,7 +464,7 @@ pyg_value_from_pyobject_with_error(GValue *value, PyObject *obj)
     {
         gchar *temp;
         if (pygi_utf8_from_py (obj, &temp)) {
-            g_value_set_string (value, temp);
+            g_value_take_string (value, temp);
             return 0;
         } else {
             /* also allows setting anything implementing __str__ */
@@ -498,7 +475,7 @@ pyg_value_from_pyobject_with_error(GValue *value, PyObject *obj)
                 return -1;
             if (pygi_utf8_from_py (str, &temp)) {
                 Py_DECREF (str);
-                g_value_set_string (value, temp);
+                g_value_take_string (value, temp);
                 return 0;
             }
             Py_DECREF (str);
@@ -511,8 +488,8 @@ pyg_value_from_pyobject_with_error(GValue *value, PyObject *obj)
         else if (PyObject_TypeCheck(obj, &PyGPointer_Type) &&
                 G_VALUE_HOLDS(value, ((PyGPointer *)obj)->gtype))
             g_value_set_pointer(value, pyg_pointer_get(obj, gpointer));
-        else if (PYGLIB_CPointer_Check(obj))
-            g_value_set_pointer(value, PYGLIB_CPointer_GetPointer(obj, NULL));
+        else if (PyCapsule_CheckExact (obj))
+            g_value_set_pointer(value, PyCapsule_GetPointer (obj, NULL));
         else if (G_VALUE_HOLDS_GTYPE (value))
             g_value_set_gtype (value, pyg_type_from_object (obj));
         else {
@@ -568,8 +545,8 @@ pyg_value_from_pyobject_with_error(GValue *value, PyObject *obj)
         }
         else if ((bm = pyg_type_lookup(G_VALUE_TYPE(value))) != NULL)
             return bm->tovalue(value, obj);
-        else if (PYGLIB_CPointer_Check(obj))
-            g_value_set_boxed(value, PYGLIB_CPointer_GetPointer(obj, NULL));
+        else if (PyCapsule_CheckExact (obj))
+            g_value_set_boxed(value, PyCapsule_GetPointer (obj, NULL));
         else {
             PyErr_SetString(PyExc_TypeError, "Expected Boxed");
             return -1;
@@ -582,7 +559,7 @@ pyg_value_from_pyobject_with_error(GValue *value, PyObject *obj)
         if (G_IS_PARAM_SPEC (pygobject_get (obj)))
             g_value_set_param(value, G_PARAM_SPEC (pygobject_get (obj)));
         else if (pyg_param_spec_check (obj))
-            g_value_set_param(value, PYGLIB_CPointer_GetPointer(obj, NULL));
+            g_value_set_param(value, PyCapsule_GetPointer (obj, NULL));
         else {
             PyErr_SetString(PyExc_TypeError, "Expected ParamSpec");
             return -1;
